@@ -41,12 +41,13 @@ namespace Banshee
             "Aaron Bockover",
             "Dan Winship",
             "Hans Petter Jansson",
-            "James Wilcox",
+            "James Willcox",
             "Chris Lahey",
             "Ben Maurer",
             "Larry Ewing",
             "Miguel de Icaza",
-            "Ulas Sahin"
+            "Ulas Sahin",
+            "Do\u011facan G\u00fcney"
         };
         
         private static string [] Artists = {
@@ -66,6 +67,7 @@ namespace Banshee
             new Translator("Vincent van Adrighem", "Dutch"),
             new Translator("\u00D8ivind Hoel", "Norwegian Bokm\u00E5l"),
             new Translator("Marco Carvalho", "Brazilian Portuguese"),
+			new Translator("Christian Rose", "Swedish")
         };
     
         private class Translator : IComparable
