@@ -1,4 +1,3 @@
-/* -*- Mode: csharp; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*- */
 /***************************************************************************
  *  IpodCore.cs
  *
@@ -188,7 +187,7 @@ namespace Banshee
         {
             this.device = device;
             user_event = new ActiveUserEvent(Catalog.GetString("Syncing iPod"));
-            user_event.Icon = Gdk.Pixbuf.LoadFromResource(IpodMisc.GetIconString(device) + "-24.png");
+            user_event.Icon = IpodMisc.GetIcon(device, 24);
             
             encodeProfile = PipelineProfile.GetConfiguredProfile("Ipod", "mp3,aac,mp4,m4a,m4p");
         }
@@ -368,14 +367,19 @@ namespace Banshee
                     break;
                 }
             }
+
+            foreach (Song song in device.SongDatabase.Songs) {
+                if (song.FileName == null)
+                    throw new ApplicationException (String.Format ("Song '{0}' has no filename", song.Title));
+            }
             
             if(!doUpdate) {
                 return;
             }
-                
+            
             ThreadPool.QueueUserWorkItem(ThreadedSave);
         }
-        
+		
         private void ThreadedSave(object o)
         {
             device.SongDatabase.SaveStarted += delegate(object o, EventArgs args) 
@@ -387,20 +391,21 @@ namespace Banshee
             {
                 user_event.Dispose();
             };
-            
-            device.SongDatabase.SaveProgressChanged += delegate(SongDatabase db, Song song, 
-                double currentPercent, int completed, int total)
-            {
-                user_event.Message = song.Artist + " - " + song.Title;
-                
-                double fraction = (double)completed / (double)total;
-                fraction += (1.0 / (double)total) * currentPercent;
-                
-                user_event.Progress = fraction;
+			
+            device.SongDatabase.SaveProgressChanged += delegate(object o, SaveProgressArgs args)
+	    {
+                user_event.Message = args.CurrentSong.Artist + " - " + args.CurrentSong.Title;
+                user_event.Progress = args.TotalProgress;
             };
             
-            device.SongDatabase.Save();
-            EmitSyncCompleted();
+            try {
+                device.SongDatabase.Save();
+            } catch (Exception e) {
+                Console.Error.WriteLine (e);
+                Core.Log.PushError ("Failed to sync iPod", e.Message);
+            } finally {
+                EmitSyncCompleted();
+            }
         }    
     }
     
@@ -494,21 +499,50 @@ namespace Banshee
            return (double)device.VolumeUsed / (double)device.VolumeSize;
         }
         
-        public static string GetIconString(Device device)
+        private static Hashtable ipod_icon_table = new Hashtable();
+        
+        public static Gdk.Pixbuf GetIcon(Device device, int size)
         {
-            switch(device.Model) {
-                case DeviceModel.Color:
-                case DeviceModel.ColorU2:
-                case DeviceModel.NanoWhite:
-                case DeviceModel.NanoBlack:
+            return GetIcon(device.Model, size);
+        }
+        
+        public static Gdk.Pixbuf GetIcon(DeviceModel model, int size)
+        {
+            string prefix = "portable-media-";
+            string id = null;
+            
+            switch(model) {
+                case DeviceModel.Color: id = "ipod-standard-color"; break;
+                case DeviceModel.ColorU2: id = "ipod-U2-color"; break;
+                case DeviceModel.Regular: id = "ipod-standard-monochrome"; break;
+                case DeviceModel.RegularU2: id = "ipod-U2-monochrome"; break;
+                case DeviceModel.Mini: id = "ipod-mini-silver"; break;
+                case DeviceModel.MiniBlue: id = "ipod-mini-blue"; break;
+                case DeviceModel.MiniPink: id = "ipod-mini-pink"; break;
+                case DeviceModel.MiniGreen: id = "ipod-mini-green"; break;
+                case DeviceModel.MiniGold: id = "ipod-mini-gold"; break;
+                case DeviceModel.Shuffle: id = "ipod-shuffle"; break;
+                case DeviceModel.NanoWhite: id = "ipod-nano-white"; break;
+                case DeviceModel.NanoBlack: id = "ipod-nano-black"; break;
                 case DeviceModel.VideoWhite:
                 case DeviceModel.VideoBlack:
-                    return "ipod-color";
-                case DeviceModel.Shuffle:
-                    return "ipod-shuffle";
+                    id = "ipod-standard-color";
+                    break;
                 default:
-                    return "ipod-regular";
+                    id = "ipod-standard-monochrome";
+                    break;
             }
+            
+            string path = ConfigureDefines.ICON_THEME_DIR 
+                + String.Format("{0}x{0}", size)
+                + Path.DirectorySeparatorChar
+                + "extras" 
+                + Path.DirectorySeparatorChar
+                + "devices" + 
+                + Path.DirectorySeparatorChar
+                + prefix + id + ".png";
+                
+            return new Gdk.Pixbuf(path);
         }
     }
 }
