@@ -39,6 +39,7 @@ using System.IO;
 
 using Sql;
 using Banshee.Logging;
+using Banshee.Widgets;
 
 namespace Banshee
 {
@@ -80,6 +81,12 @@ namespace Banshee
         private Button ipodPropertiesButton;
         private Button ipodEjectButton;
             
+        private MultiStateToggleButton repeat_toggle_button;
+        private MultiStateToggleButton shuffle_toggle_button;
+                
+        private EventBox syncing_container;
+        private Gtk.Image ipod_syncing_image = new Gtk.Image();
+            
         private CoverArtThumbnail cover_art;
         
         private bool incrementedCurrentSongPlayCount;
@@ -227,14 +234,7 @@ namespace Banshee
             ImageRip.SetFromStock("media-rip", IconSize.LargeToolbar);
 
             gxml["ButtonBurn"].Visible = true;
-                
-            ((Gtk.Image)gxml["ImageShuffle"]).Pixbuf = 
-                Gdk.Pixbuf.LoadFromResource("media-shuffle.png");
-            ((Gtk.Image)gxml["ImageRepeat"]).Pixbuf = 
-                Gdk.Pixbuf.LoadFromResource("media-repeat.png");
-            ((Gtk.Image)gxml["ImageIpodSync"]).Pixbuf = 
-                Gdk.Pixbuf.LoadFromResource("ipod-regular-24.png");
-            
+      
             // Header
             headerNotebook = new SimpleNotebook();
             headerNotebook.Show();
@@ -255,6 +255,10 @@ namespace Banshee
             volumeButton.Visible = true;
             volumeButton.VolumeChanged += 
                 new VolumeButton.VolumeChangedHandler(OnVolumeScaleChanged);
+                
+            // Next/Previous Buttons
+            
+            (gxml["ButtonPrevious"] as Button).ButtonPressEvent += OnButtonPreviousPressed;
 
             // Cover Art Thumbnail
             cover_art = new CoverArtThumbnail(36);
@@ -292,6 +296,7 @@ namespace Banshee
             ((Gtk.ScrolledWindow)gxml["LibraryContainer"]).Add(playlistView);
             playlistView.Show();
             playlistModel.Updated += OnPlaylistUpdated;
+            playlistView.KeyPressEvent += OnPlaylistViewKeyPressEvent;
             playlistView.ButtonPressEvent += OnPlaylistViewButtonPressEvent;
             playlistView.MotionNotifyEvent += OnPlaylistViewMotionNotifyEvent;
             playlistView.ButtonReleaseEvent += OnPlaylistViewButtonReleaseEvent;
@@ -367,10 +372,36 @@ namespace Banshee
             ((HBox)gxml["PlaylistHeaderBox"]).PackStart(searchEntry, 
                 false, false, 0);
                 
+            // Repeat/Shuffle buttons
+            
+            repeat_toggle_button = new MultiStateToggleButton();
+            repeat_toggle_button.AddState(typeof(RepeatNoneToggleState), gxml["ItemRepeatNone"] as RadioMenuItem);
+            repeat_toggle_button.AddState(typeof(RepeatAllToggleState), gxml["ItemRepeatAll"] as RadioMenuItem);
+            repeat_toggle_button.AddState(typeof(RepeatSingleToggleState), gxml["ItemRepeatSingle"] as RadioMenuItem);
+            repeat_toggle_button.Relief = ReliefStyle.None;
+            repeat_toggle_button.ShowLabel = false;
+            repeat_toggle_button.Changed += delegate(object o, ToggleStateChangedArgs args) {
+                HandleRepeatToggleButton();
+            };
+            repeat_toggle_button.ShowAll();
+                
+            shuffle_toggle_button = new MultiStateToggleButton();
+            shuffle_toggle_button.AddState(typeof(ShuffleDisabledToggleState), gxml["ItemShuffle"] as CheckMenuItem, false);
+            shuffle_toggle_button.AddState(typeof(ShuffleEnabledToggleState), gxml["ItemShuffle"] as CheckMenuItem, true);
+            shuffle_toggle_button.Relief = ReliefStyle.None;
+            shuffle_toggle_button.ShowLabel = false;
+            shuffle_toggle_button.Changed += delegate(object o, ToggleStateChangedArgs args) {
+                HandleShuffleToggleButton();
+            };
+            shuffle_toggle_button.ShowAll();
+            
+            (gxml["ShuffleButtonContainer"] as Container).Add(shuffle_toggle_button);
+            (gxml["RepeatButtonContainer"] as Container).Add(repeat_toggle_button);
+                
             toolTips = new Tooltips();
             SetTip(gxml["ButtonNewPlaylist"], Catalog.GetString("Create New Playlist"));
-            SetTip(gxml["ToggleButtonShuffle"], Catalog.GetString("Toggle Shuffle Playback Mode"));
-            SetTip(gxml["ToggleButtonRepeat"], Catalog.GetString("Toggle Repeat Playback Mode"));
+            SetTip(shuffle_toggle_button, Catalog.GetString("Shuffle Playback Mode"));
+            SetTip(repeat_toggle_button, Catalog.GetString("Repeat Playback Mode"));
             SetTip(gxml["ButtonTrackProperties"], Catalog.GetString("View Selected Song Information"));
             SetTip(gxml["ButtonBurn"], Catalog.GetString("Write Selection to CD"));
             SetTip(gxml["ButtonRip"], Catalog.GetString("Import CD into Library"));
@@ -433,13 +464,16 @@ namespace Banshee
             }
             
             try {
-                ((ToggleButton)gxml["ToggleButtonShuffle"]).Active = (bool)Core.GconfClient.Get(
-                    GConfKeys.PlaylistShuffle);
-                ((ToggleButton)gxml["ToggleButtonRepeat"]).Active = (bool)Core.GconfClient.Get(
-                    GConfKeys.PlaylistRepeat);
-            } catch(GConf.NoSuchKeyException) {
-                // Default, set in glade file
-            }
+                repeat_toggle_button.ActiveState = 
+                    Type.GetType((string)Core.GconfClient.Get(GConfKeys.PlaylistRepeat));
+                HandleRepeatToggleButton();
+            } catch(Exception) {}
+            
+            try {
+                shuffle_toggle_button.ActiveState = 
+                    Type.GetType((string)Core.GconfClient.Get(GConfKeys.PlaylistShuffle));
+                HandleShuffleToggleButton();
+            } catch(Exception) {}
             
             try {
                 SourceSplitter.Position = (int)Core.GconfClient.Get(GConfKeys.SourceViewWidth);
@@ -543,6 +577,13 @@ namespace Banshee
         {
             startupLoadReady = true;
             
+            if(Core.ArgumentQueue.Contains("play")) {
+                GLib.Timeout.Add(1500, delegate {
+                    PlayPause();
+                    return false;
+                });
+            }
+            
             if(Core.Library.Tracks.Count <= 0) {
                 Application.Invoke(delegate { 
                     PromptForImport();
@@ -577,7 +618,7 @@ namespace Banshee
             if(trayIcon != null && Core.Instance.Player.Playing) {
                 trayIcon.Tooltip = activeTrackInfo.DisplayArtist + " - " 
                     + activeTrackInfo.DisplayTitle + "\n"
-                    + Catalog.GetString("Position:") + text;
+                    + Catalog.GetString("Position: ") + text;
             }
         }
           
@@ -604,7 +645,8 @@ namespace Banshee
             trackInfoHeader.Artist = ti.DisplayArtist;
             trackInfoHeader.Title = ti.DisplayTitle;
             
-            cover_art.Track = ti;
+            cover_art.Label = String.Format("{0} - {1}", ti.Artist, ti.Album);
+            cover_art.FileName = ti.CoverArtFileName;
             
             if(trayIcon != null) {
                 trayIcon.Tooltip = ti.DisplayArtist + " - " + ti.DisplayTitle;
@@ -694,10 +736,27 @@ namespace Banshee
                         handled = true;
                     } 
                     break;
+                case Gdk.Key.space:
+                    if(!searchEntry.HasFocus) {
+                        PlayPause();
+                        handled = true;
+                    }
+                    break;
             }
             
             args.RetVal = handled;
         }
+        
+        [GLib.ConnectBefore]
+        private void OnPlaylistViewKeyPressEvent(object o, KeyPressEventArgs args)
+        {
+            switch(args.Event.Key) {
+                case Gdk.Key.Return:
+                    playlistView.PlaySelected();
+                    args.RetVal = true;
+                    break;
+            }
+        }       
               
         // ---- Tray Event Handlers ----
       
@@ -758,11 +817,19 @@ namespace Banshee
         // ---- Playback Event Handlers ----
 
         private void OnButtonPlayPauseClicked(object o, EventArgs args)
+        {
+            PlayPause();
+        }
+        
+        public void PlayPause()
         {
             if(Core.Instance.Player.Loaded) {
                 TogglePlaying();
             } else {
-                playlistView.PlaySelected();
+                if(!playlistView.PlaySelected()) {
+                    playlistModel.Advance();
+                    playlistView.UpdateView();
+                }
             }
         }
         
@@ -780,7 +847,11 @@ namespace Banshee
         
         private void OnButtonPreviousClicked(object o, EventArgs args)
         {
-            Previous();
+            if(Core.Instance.Player.Position < 3) {
+                Previous();
+            } else {
+                Core.Instance.Player.Position = 0;
+            }
         }
         
         private void OnButtonNextClicked(object o, EventArgs args)
@@ -788,6 +859,15 @@ namespace Banshee
             Next();
         }
         
+        [GLib.ConnectBefore]
+        private void OnButtonPreviousPressed(object o, ButtonPressEventArgs args)
+        {
+            if((args.Event.State & Gdk.ModifierType.ShiftMask) != 0) {
+                Previous();
+                args.RetVal = true;
+            }
+        }
+        
         private void OnVolumeScaleChanged(int volume)
         {
             Core.Instance.Player.Volume = (ushort)volume;
@@ -915,6 +995,7 @@ namespace Banshee
             return false;
         }
         
+        [GLib.ConnectBeforeAttribute]
         private void OnScaleTimeMoveSlider(object o, EventArgs args)
         {
             SetPositionLabel((long)ScaleTime.Value);
@@ -1150,6 +1231,7 @@ namespace Banshee
                 gxml["IpodContainer"].ShowAll();
                 IpodSource ipodSource = source as IpodSource;
                 ipodSyncButton.Visible = ipodSource.Device.CanWrite;
+                ((Gtk.Image)gxml["ImageIpodSync"]).Pixbuf = IpodMisc.GetIcon(ipodSource.Device, 24);
             } else {
                 gxml["IpodContainer"].Visible = false;
             }     
@@ -1300,7 +1382,7 @@ namespace Banshee
                     Catalog.GetString("Synchronize Library"));
                     md.AddButton(Catalog.GetString("Save Manual Changes"), 
                         Gtk.ResponseType.Apply, true);
-                md.Image = Gdk.Pixbuf.LoadFromResource("ipod-48.png");
+                md.Image = IpodMisc.GetIcon(ipodSource.Device, 48);
                 md.Icon = md.Image;
                 switch(md.Run()) {
                     case (int)ResponseType.Ok:
@@ -1326,7 +1408,7 @@ namespace Banshee
                     "with your Banshee library? This will <b>erase</b> the contents of " +
                     "your iPod and then copy the contents of your Banshee library."),
                     Catalog.GetString("Synchronize Library"));
-                md.Image = Gdk.Pixbuf.LoadFromResource("ipod-48.png");
+                md.Image = IpodMisc.GetIcon(ipodSource.Device, 48);
                 md.Icon = md.Image;
                 switch(md.Run()) {
                     case (int)ResponseType.Ok:
@@ -1357,6 +1439,7 @@ namespace Banshee
                 gxml["IpodSyncButton"].Sensitive = false;
                 gxml["SearchLabel"].Sensitive = false;
                 searchEntry.Sensitive = false;
+                ipod_syncing_image.Pixbuf = IpodMisc.GetIcon((playlistModel.Source as IpodSource).Device, 128);
             }
         }
         
@@ -1380,28 +1463,23 @@ namespace Banshee
             playlistView.QueueDraw();
         }
     
-        private void OnToggleButtonShuffleToggled(object o, EventArgs args)
+        private void HandleRepeatToggleButton()
         {
-            ToggleButton t = (ToggleButton)o;
-            playlistModel.Shuffle = t.Active;
-            Core.GconfClient.Set(GConfKeys.PlaylistShuffle, 
-                t.Active);
-                
-            if(trayIcon != null)
-                ((Gtk.Image)trayIcon.ShuffleItem.Image).SetFromStock(
-                    t.Active ? "gtk-yes" : "gtk-no", IconSize.Menu);
+            if(repeat_toggle_button.ActiveState == typeof(RepeatAllToggleState)) {
+                playlistModel.Repeat = RepeatMode.All;
+            } else if(repeat_toggle_button.ActiveState == typeof(RepeatSingleToggleState)) {
+                playlistModel.Repeat = RepeatMode.Single;
+            } else {
+                playlistModel.Repeat = RepeatMode.None;
+            }
+            
+            Core.GconfClient.Set(GConfKeys.PlaylistRepeat, repeat_toggle_button.ActiveState.ToString());
         }
         
-        private void OnToggleButtonRepeatToggled(object o, EventArgs args)
+        private void HandleShuffleToggleButton()
         {
-            ToggleButton t = (ToggleButton)o;
-            playlistModel.Repeat = t.Active;
-            Core.GconfClient.Set(GConfKeys.PlaylistRepeat, 
-                t.Active);
-                
-            if(trayIcon != null)
-                ((Gtk.Image)trayIcon.RepeatItem.Image).SetFromStock(
-                    t.Active ? "gtk-yes" : "gtk-no", IconSize.Menu);
+            playlistModel.Shuffle = shuffle_toggle_button.ActiveState == typeof(ShuffleEnabledToggleState);
+            Core.GconfClient.Set(GConfKeys.PlaylistShuffle, shuffle_toggle_button.ActiveState.ToString());
         }
         
         private void OnPlaylistUpdated(object o, EventArgs args)
@@ -1755,16 +1833,20 @@ namespace Banshee
             
             InputDialog input;
             
-            if(source.Type == SourceType.Playlist)
-             input = new InputDialog(
-                Catalog.GetString("Rename Playlist"),
-                Catalog.GetString("Enter new playlist name"), 
-                "playlist-icon-large.png", source.Name);
-            else
-              input = new InputDialog(
-                  Catalog.GetString("Rename iPod"),
-                  Catalog.GetString("Enter new name for your iPod"),
-                  "ipod-48.png", source.Name);
+            if(source.Type == SourceType.Playlist) {
+                input = new InputDialog(
+                    Catalog.GetString("Rename Playlist"),
+                    Catalog.GetString("Enter new playlist name"), 
+                    Gdk.Pixbuf.LoadFromResource("playlist-icon-large.png"), source.Name);
+            } else if(source.Type == SourceType.Ipod) {
+                IpodSource ipod_source = source as IpodSource;
+                input = new InputDialog(
+                    Catalog.GetString("Rename iPod"),
+                    Catalog.GetString("Enter new name for your iPod"),
+                    IpodMisc.GetIcon(ipod_source.Device, 48), source.Name);
+            } else {
+                return;
+            }
                 
             string newName = input.Execute();
             if(newName != null)
@@ -2331,36 +2413,42 @@ namespace Banshee
             
             if(alignment.Child == playlist_container) {
                 return;
-            } else if(alignment.Child == syncing_label) {
-                alignment.Remove(syncing_label);
+            } else if(alignment.Child == syncing_container) {
+                alignment.Remove(syncing_container);
             }
             
             alignment.Add(playlist_container);
             alignment.ShowAll();
         }
         
-        
-        private Label syncing_label;
-        
         private void ShowSyncingView()
         {
             Alignment alignment = gxml["LibraryAlignment"] as Alignment;
             ScrolledWindow playlist_container = gxml["LibraryContainer"] as ScrolledWindow;
             
-            if(alignment.Child == syncing_label) {
+            if(alignment.Child == syncing_container) {
                 return;
             } else if(alignment.Child == playlist_container) {
                 alignment.Remove(playlist_container);
             }
             
-            if(syncing_label == null) {
-                syncing_label = new Label();
+            if(syncing_container == null) {
+                syncing_container = new EventBox();
+                HBox syncing_box = new HBox();
+                syncing_container.Add(syncing_box);
+                syncing_box.Spacing = 20;
+                syncing_box.PackStart(ipod_syncing_image, false, false, 0);
+                Label syncing_label = new Label();
+                                                
+                syncing_container.ModifyBg(StateType.Normal, new Color(0, 0, 0));
+                syncing_label.ModifyFg(StateType.Normal, new Color(160, 160, 160));
+            
                 syncing_label.Markup = "<big><b>" + GLib.Markup.EscapeText(
-                    Catalog.GetString("Syncing your iPod, Please Wait...")) + "</b></big>";
-                syncing_label.Show();
+                    Catalog.GetString("Synchronizing your iPod, Please Wait...")) + "</b></big>";
+                syncing_box.PackStart(syncing_label, false, false, 0);
             }
             
-            alignment.Add(syncing_label);
+            alignment.Add(syncing_container);
             alignment.ShowAll();
         }
     }
