@@ -35,6 +35,12 @@ using Sql;
 
 namespace Banshee
 {
+    public enum RepeatMode {
+        None,
+        All,
+        Single
+    }
+
     public class PlaylistModel : ListStore, IPlaybackModel
     {
         private static int uid;
@@ -44,7 +50,7 @@ namespace Banshee
         private bool trackInfoQueueLocked = false;
         private TreeIter playingIter;
         
-        private bool repeat = false;
+        private RepeatMode repeat = RepeatMode.None;
         private bool shuffle = false;
         
         private bool canUpdate;
@@ -293,8 +299,14 @@ namespace Banshee
             }
             
             if(currentPath == null) {
-                if(GetIterFirst(out nextIter))
-                    PlayIter(nextIter);
+                if(shuffle) {
+                    if(!GetRandomTrackIter(out nextIter))
+                        return;
+                } else if(!GetIterFirst(out nextIter)) {
+                    return;
+                }
+
+                PlayIter(nextIter);
                 return;
             }
         
@@ -308,13 +320,14 @@ namespace Banshee
             currentTrack = PathTrackInfo(currentPath);
             currentIter = playingIter;
         
-            if(forward) {
-                if(lastTrack && repeat) {
+            if(repeat == RepeatMode.Single) {
+                nextIter = currentIter;
+            } else if(forward) {
+                if(lastTrack && repeat == RepeatMode.All) {
                     if(!IterNthChild(out nextIter, 0))
                         return;
                 } else if(shuffle) {
-                    int randIndex = Core.Instance.Random.Next(0, Count());
-                    if(!IterNthChild(out nextIter, randIndex))
+                    if(!GetRandomTrackIter(out nextIter))
                         return;
                 } else {                
                     currentPath.Next();                
@@ -362,6 +375,12 @@ namespace Banshee
     
             return -1;
         }
+
+        private bool GetRandomTrackIter(out TreeIter iter)
+        {
+            int randIndex = Core.Instance.Random.Next(0, Count());
+            return IterNthChild(out iter, randIndex);
+        }
         
         public void ClearModel()
         {
@@ -456,7 +475,7 @@ namespace Banshee
             }
         }
         
-        public bool Repeat {
+        public RepeatMode Repeat {
             set {
                 repeat = value;
             }
