@@ -1,4 +1,3 @@
-/* -*- Mode: csharp; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*- */
 /***************************************************************************
  *  PlaylistView.cs
  *
@@ -347,10 +346,10 @@ namespace Banshee
         {
             CellRendererToggle toggle = (CellRendererToggle)cell;
             AudioCdTrackInfo ti = model.IterTrackInfo(iter) as AudioCdTrackInfo;
-			if (ti != null)
-				toggle.Active = ti.CanRip;
-			else
-				toggle.Active = false;
+            if (ti != null)
+                toggle.Active = ti.CanRip;
+            else
+                toggle.Active = false;
         }
         
         protected void SyncCellInd(TreeViewColumn tree_column,
@@ -452,13 +451,13 @@ namespace Banshee
         {
             model.PlayPath(path);
             QueueDraw();
-            ScrollToCell(model.PlayingPath, null, true, 0.5f, 0.0f);
+            ScrollToPlaying();
         }
         
         public void UpdateView()
         {
             QueueDraw();
-            ScrollToCell(model.PlayingPath, null, true, 0.5f, 0.5f);
+            ScrollToPlaying();
         }
         
         public void ThreadedQueueDraw()
@@ -467,6 +466,21 @@ namespace Banshee
                 QueueDraw();
             });
         }
+
+        public void ScrollToPlaying()
+        {
+            Gdk.Rectangle cellRect = GetCellArea (model.PlayingPath, Columns[0]);
+
+            Point point = new Point ();
+            WidgetToTreeCoords (cellRect.Left, cellRect.Top, out point.X, out point.Y);
+            cellRect.Location = point;
+
+            // we only care about vertical bounds
+            if (cellRect.Location.Y < VisibleRect.Location.Y ||
+                cellRect.Location.Y + cellRect.Size.Height > VisibleRect.Location.Y + VisibleRect.Size.Height) {
+                ScrollToCell(model.PlayingPath, null, true, 0.5f, 0.0f);
+            }
+        }
         
         //QueueDraw();
         //ScrollToCell(playingPath, null, true, 0.5f, 0.0f);
@@ -666,18 +680,19 @@ namespace Banshee
         */
         
         
-        public void PlaySelected()
+        public bool PlaySelected()
         {
             TreeIter selIter;
             
             try {
                 if(!model.GetIter(out selIter, Selection.GetSelectedRows()[0]))
-                    return;
+                    return false;
             } catch(Exception) {
-                return;
+                return false;
             }
             
             model.PlayIter(selIter);
+            return true;
         }
         
         public void AddSelectedToPlayList(string name)
